from ThodarKuri.Grammer.SyntaxParser import ThodarkuriParser
import pytest

def test_Template1():

    NodeList = [
             " R[Var_new_1] ",
             " R[Var_new_3] => files1.html ",
             " ( R[Var_new_4] => files2.html )* "
             ]

    NodeResult = [
             { 'VAR' : 'Var_new_1' },
             { 'Var_new_3' : 'files1.html' },
             [{ 'Var_new_4' : 'files2.html' },]
             ]

    for i in range(len(NodeList)):
    	# print(ThodarkuriParser(NodeList[i]));
        assert(ThodarkuriParser(NodeList[i]) == NodeResult[i]);


from lark import Lark, Transformer, v_args

TK_Grammer = """
        ?start: var
              | dictvar
              | listvar

        ?listvar: "(" dictvar ")*" -> getdictvaraslist
        
        ?dictvar: "R["VAR"] => "FILE -> getvarasdict
        
        ?var: "R["VAR"]" -> getvar

        VAR: /[a-zA-Z_]\\w*/
        FILE: /[a-zA-Z_]\\w*.[a-zA-Z_]\\w*/

        %import common.WS_INLINE
        %ignore WS_INLINE 

    """

@v_args(inline=True)    
class TK_Parser(Transformer):

    def __init__(self):
        pass

    def getvar(self, var):

        Dict = { var.type : var.value }
        return Dict

    def getvarasdict(self, var, file):


        Dict = { var.value: file.value }
        return Dict

    def getdictvaraslist(self, dictvar):

        List = [dictvar];
        return List

TK_Template = Lark(TK_Grammer, parser='lalr', transformer=TK_Parser())
ThodarkuriParser = TK_Template.parse



from ThodarKuri.Filler import FillerTemplateEngine
import json
import copy
import pytest

@pytest.fixture
def DefaultRegex1():
    TE = FillerTemplateEngine();
    return TE;

def test_DefaultCase1(DefaultRegex1): 
     
    with open('gen_files/Settings.json') as config:
        Settings = json.load(config)

    with open('ref_files/PlotDetails.json') as Details:
        PlotDetails = json.load(Details)

    Settings['Title'] = 'Template Engine Demo';

    Settings['Plot']['Plot_Description'] = 'Horizontal Bar Chart';
    Settings['Plot']['Plot_Orientation'] = 'horizontal';

    filtered_Lists = [bar for bar in PlotDetails if bar['size'] < 100];

    bar_dict = Settings['Plot']['Bar'][0]
    Settings['Plot']['Bar'] = [];
    for bar in filtered_Lists:
        bar_dict['name'] = bar['name'];
        bar_dict['size'] = bar['size'];
        bar_dict['description'] = bar['description'];
        # deep copy to be used when handling copy of lists and dict
        Settings['Plot']['Bar'].append(copy.deepcopy(bar_dict));
    
    # print(json.dumps(Settings, sort_keys=True, indent=4));

    Filler = DefaultRegex1;
    FilledString = Filler.FillEntryPoint(Settings, "html_plot/template_index.html", "html_plot/index.html");

    ReferenceFile = open("ref_files/index.html", 'r').readlines();
    CreatedFile = open("html_plot/index.html", 'r').readlines();
    assert len(ReferenceFile) == len(CreatedFile);
    for line_no in range(len(CreatedFile)):
        assert ReferenceFile[line_no] == CreatedFile[line_no]

@pytest.fixture
def SpecialRegex1():
    TE = FillerTemplateEngine(("_StArT_", "_eNd_"), "_StArT_self.FUNC_CALL()_eNd_");
    return TE;

@pytest.fixture
def SpecialRegex2():
    TE = FillerTemplateEngine(("<<", ">>"), "<<self.FUNC_CALL()>>");
    return TE;

@pytest.fixture
def SpecialRegex3():
    TE = FillerTemplateEngine(("\\[\\[", "\\]\\]"), "[[self.FUNC_CALL()]]");
    return TE;

@pytest.fixture
def SpecialRegex4():
    TE = FillerTemplateEngine(("{\\[<{\\[", "}\\]>}\\]"), "{[<{[self.FUNC_CALL()}]>}]");
    return TE;

def test_SpecialCase1(SpecialRegex1, SpecialRegex2, SpecialRegex3, SpecialRegex4): 
     
    for filex in ([1,2,3,4]):

        if(filex==1):Filler = SpecialRegex1;
        if(filex==2):Filler = SpecialRegex2;
        if(filex==3):Filler = SpecialRegex3;
        if(filex==4):Filler = SpecialRegex4;

        with open(f'gen_files/Settings - SR{filex}.json') as config:
            Settings = json.load(config)

        with open('ref_files/PlotDetails.json') as Details:
            PlotDetails = json.load(Details)

        Settings['Title'] = 'Template Engine Demo';

        Settings['Plot']['Plot_Description'] = 'Horizontal Bar Chart';
        Settings['Plot']['Plot_Orientation'] = 'horizontal';

        filtered_Lists = [bar for bar in PlotDetails if bar['size'] < 100];

        bar_dict = Settings['Plot']['Bar'][0]
        Settings['Plot']['Bar'] = [];
        for bar in filtered_Lists:
            bar_dict['name'] = bar['name'];
            bar_dict['size'] = bar['size'];
            bar_dict['description'] = bar['description'];
            # deep copy to be used when handling copy of lists and dict
            Settings['Plot']['Bar'].append(copy.deepcopy(bar_dict));
        
        # print(json.dumps(Settings, sort_keys=True, indent=4));

        FilledString = Filler.FillEntryPoint(Settings, f'html_plot - SR{filex}/template_index.html', f'html_plot - SR{filex}/index.html');

        ReferenceFile = open('ref_files/index.html', 'r').readlines();
        CreatedFile = open(f'html_plot - SR{filex}/index.html', 'r').readlines();
        assert len(ReferenceFile) == len(CreatedFile);
        for line_no in range(len(CreatedFile)):
            assert ReferenceFile[line_no] == CreatedFile[line_no]


import re
import os
import json
from .Grammer.SyntaxParser import ThodarkuriParser

class ParserTemplateEngine():

    def __init__(self, RegexEdges=("{{", "}}"), FuncCallTemplate="{{self.FUNC_CALL()}}"):

        RegexPattern=f"{RegexEdges[0]} *(?:(?!{RegexEdges[1]}).)*{RegexEdges[1]}";
        func_call = [FuncCallTemplate[m.start(0):m.end(0)] for m in re.finditer(RegexPattern, FuncCallTemplate)][0]
        assert func_call == FuncCallTemplate, "RegexPattern and FuncCallTemplate not matching"
        
        lst = re.split("self.FUNC_CALL\\(\\)", func_call.replace(" ", ""));
        cap, cap_len, shoe ,shoe_len = tuple([ lst[0], len(lst[0]), lst[1], len(lst[1]) ]);
        assert FuncCallTemplate[cap_len:-shoe_len].strip() == "self.FUNC_CALL()", """     Second Parameter has 3 parts, \n <LEADING_EDGE><FUNC_CALL><TRAILING_EDGE>""";
        
        self.__pattern = RegexPattern.replace("*self.","*#* *self.");
        self.__LeadTrailSpecs = [ ( cap, cap_len ), ( shoe, -shoe_len )];

    def __ParseNode(self, MapDict, node):

        RetVal = ThodarkuriParser(node)

        if(str(type(RetVal)) == "<class 'list'>"):
            for x,y in RetVal[0].items():
                MapDict[x]=[self.__ParseContent(y)];
                    
        if(str(type(RetVal)) == "<class 'dict'>"):
            if('VAR' in RetVal.keys()):
                MapDict[RetVal['VAR']] = None;
            else:
                for x,y in RetVal.items():
                    MapDict[x]=self.__ParseContent(y);
                    
        return MapDict;


    def __ParseContent(self, TemplateName):
        
        template = open(os.path.join(self.__FolderPath, TemplateName), 'r');
        self.__content = template.read();
        template.close();

        MapDict = {};
        func_calls = [self.__content[m.start(0):m.end(0)] for m in re.finditer(self.__pattern, self.__content)];
        for x in func_calls:
            InpNode = x[self.__LeadTrailSpecs[0][1]:self.__LeadTrailSpecs[1][1]].strip();
            if(not(InpNode.startswith('#'))):
                MapDict = self.__ParseNode(MapDict, InpNode);
            
        return MapDict;

    def ParseEntryPoint(self, TemplateName, DebugTokens = False):

        TemplateName = os.path.abspath(TemplateName);
        self.__FolderPath = os.path.dirname(TemplateName);
        MapDict = self.__ParseContent(TemplateName);
        if(DebugTokens): print(json.dumps(MapDict, sort_keys=True, indent=4));
        return MapDict;
        


import re
import os
import json
from .Grammer.SyntaxParser import ThodarkuriParser

class FillerTemplateEngine():

    def __init__(self, RegexEdges=("{{", "}}"), FuncCallTemplate="{{self.FUNC_CALL()}}"):

        RegexPattern=f"{RegexEdges[0]} *(?:(?!{RegexEdges[1]}).)*{RegexEdges[1]}";
        func_call = [FuncCallTemplate[m.start(0):m.end(0)] for m in re.finditer(RegexPattern, FuncCallTemplate)][0]
        assert func_call == FuncCallTemplate, "RegexPattern and FuncCallTemplate not matching"
        
        lst = re.split("self.FUNC_CALL\\(\\)", func_call.replace(" ", ""));
        cap, cap_len, shoe ,shoe_len = tuple([ lst[0], len(lst[0]), lst[1], len(lst[1]) ]);
        assert FuncCallTemplate[cap_len:-shoe_len].strip() == "self.FUNC_CALL()", """     Second Parameter has 3 parts, \n <LEADING_EDGE><FUNC_CALL><TRAILING_EDGE>""";
        
        self.__pattern = RegexPattern.replace("*self.","*#* *self.");
        self.__LeadTrailSpecs = [ ( cap, cap_len ), ( shoe, -shoe_len )];

    def __FillNode(self, content, MapDict, node):

        InpNode = node[self.__LeadTrailSpecs[0][1]:self.__LeadTrailSpecs[1][1]].strip();
        RetVal = ThodarkuriParser(InpNode)
        if(str(type(RetVal)) == "<class 'list'>"):
            for x,y in RetVal[0].items():
                content = content.replace(node, ''.join([ self.__FillContent(y, each) for each in MapDict[x] ]) )
                    
        if(str(type(RetVal)) == "<class 'dict'>"):
            if('VAR' in RetVal.keys()):
                content = content.replace(node, str(MapDict[RetVal['VAR']]))
            else:
                for x,y in RetVal.items():
                    content = content.replace(node, self.__FillContent(y, MapDict[x]))

        return content;
    
    def __FillContent(self, TemplateName, MapDict):

        template = open(os.path.join(self.__FolderPath, TemplateName), 'r');
        content = template.read();
        template.close();

        func_calls = [content[m.start(0):m.end(0)] for m in re.finditer(self.__pattern, content)];
        for x in func_calls:
            InpNode = x[self.__LeadTrailSpecs[0][1]:self.__LeadTrailSpecs[1][1]].strip();
            if(InpNode.startswith('#')):
                content = content.replace(x, '');
            else:
                content = self.__FillNode(content, MapDict, x);
            
        return content;
        
    def FillEntryPoint(self, MapDict, TemplateName, FileName = None, DebugTokens = False):

        self.__FolderPath = os.path.dirname(os.path.abspath(TemplateName));
        MappedStr = self.__FillContent(os.path.basename(TemplateName), MapDict);
        if(DebugTokens): print(MappedStr);
        if(FileName != None):
            OutputFile = open(FileName, "w");
            OutputFile.write(MappedStr)
            OutputFile.close();
        return MappedStr;



from ThodarKuri.Parser import ParserTemplateEngine
import json
import pytest

@pytest.fixture
def DefaultRegex1():
    TE = ParserTemplateEngine();
    return TE;

def test_DefaultCase1(DefaultRegex1):

    Parser = DefaultRegex1;
    Settings = Parser.ParseEntryPoint("html_plot/template_index.html");

    # print(json.dumps(Settings, sort_keys=True, indent=4));

    OutputFile = open('gen_files/Settings.json', "w");
    OutputFile.write(json.dumps(Settings, sort_keys=True, indent=4))
    OutputFile.close();

    ReferenceFile = open("ref_files/Reference_Settings.json", 'r').readlines();
    CreatedFile = open("gen_files/Settings.json", 'r').readlines();
    assert len(ReferenceFile) == len(CreatedFile);
    for line_no in range(len(CreatedFile)):
        assert ReferenceFile[line_no] == CreatedFile[line_no]

@pytest.fixture
def SpecialRegex1():
    TE = ParserTemplateEngine(("_StArT_", "_eNd_"), "_StArT_self.FUNC_CALL()_eNd_");
    return TE;

@pytest.fixture
def SpecialRegex2():
    TE = ParserTemplateEngine(("<<", ">>"), "<<self.FUNC_CALL()>>");
    return TE;

@pytest.fixture
def SpecialRegex3():
    TE = ParserTemplateEngine(("\\[\\[", "\\]\\]"), "[[self.FUNC_CALL()]]");
    return TE;

@pytest.fixture
def SpecialRegex4():
    TE = ParserTemplateEngine(("{\\[<{\\[", "}\\]>}\\]"), "{[<{[self.FUNC_CALL()}]>}]");
    return TE;


def test_SpecialCase1(SpecialRegex1, SpecialRegex2, SpecialRegex3, SpecialRegex4):

    for filex in ([1,2,3,4]):

        if(filex==1):Parser = SpecialRegex1;
        if(filex==2):Parser = SpecialRegex2;
        if(filex==3):Parser = SpecialRegex3;
        if(filex==4):Parser = SpecialRegex4;

        Settings = Parser.ParseEntryPoint(f'html_plot - SR{filex}/template_index.html');

        # print(json.dumps(Settings, sort_keys=True, indent=4));

        OutputFile = open(f'gen_files/Settings - SR{filex}.json', "w");
        OutputFile.write(json.dumps(Settings, sort_keys=True, indent=4))
        OutputFile.close();

        ReferenceFile = open("ref_files/Reference_Settings.json", 'r').readlines();
        CreatedFile = open(f'gen_files/Settings - SR{filex}.json', 'r').readlines();
        assert len(ReferenceFile) == len(CreatedFile);
        for line_no in range(len(CreatedFile)):
            assert ReferenceFile[line_no] == CreatedFile[line_no]



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="ThodarKuri",
    version="0.0.2",
    description="Generic Recursive Template Engine for Parsing and Filling",
    py_modules=["ThodarKuri/Filler", "ThodarKuri/Parser"],
    package_dir={"": "SRCS"},
    packages=['ThodarKuri','ThodarKuri/Grammer'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    url="https://github.com/Palani-SN/ThodarKuri",
    author="Palani-SN",
    author_email="psn396@gmail.com",

    install_requires = [
        "blessings ~= 1.7",
        "lark >= 1.0.0",
    ],

    extras_require = {
        "dev": [
            "pytest >= 3.7",
            "check-manifest",
            "twine",
            "requests >= 2.26.0",
        ],
    },
)
